/**
 * @file                                                    THERMO_K_click.c
 * @brief                   THERMO K Example
 *
 * @par
 * Example for Thermo K click board.
 *
 * Test configuration :
 * @par
 * - <b>MCU           </b> :    FT900
 * - <b>Dev. Board    </b> :    EasyFT90x v7
 * - <b>Ext. Modules  </b> :    EasyTFT & Thermo K click
 * - <b>SW            </b> :    mikroC PRO for FT90x v2.0.0
 *
 ******************************************************************************/

#include "MCP9600.h"
#include "picture.h"
#include "tft_resources.h"

char tk_status;
char txt[10];
char lab1[10];
char old_txt[10];
char tmp_data[10];
char Temp_Data_Ready;

/*******************************************************************************
* Function Write_MCP9600_Data(RegAddr)
* ------------------------------------------------------------------------------
* Overview: Function writes Data to MCP9600
* Input: Register address, data
* Output: Nothing
*******************************************************************************/
char Write_MCP9600_Data(char RegAddr, char _data)
{
  tmp_data[0] = RegAddr;                                                        // Register address
  tmp_data[1] = _data;
  
  I2CM_Set_Slave_Address( MCP9600_I2C_ADDR );
  I2CM_Write_Bytes( tmp_data, 2 );
}

/*******************************************************************************
* Function Read_MCP9600_Data(RegAddr)
* ------------------------------------------------------------------------------
* Overview: Function gets Data from MCP9600
* Input: Register address
* Output: Value from desired register
*******************************************************************************/
char Read_MCP9600_Data(char RegAddr)
{
  tmp_data[0] = RegAddr;                                                        // Register address


  I2CM_Set_Slave_Address( MCP9600_I2C_ADDR );
  I2CM_Write_Bytes( tmp_data, 1 );
  Delay_us(50);
  I2CM_Read_Bytes( tmp_data, 1 );

  return tmp_data[0];                                                           // Return MCP9600 Data value
}

/*******************************************************************************
* Function Read_MCP9600_Status
* ------------------------------------------------------------------------------
* Overview: Function gets Data from MCP9600
* Input: Register address
* Output: Value from desired register
*******************************************************************************/
char Read_MCP9600_Status()
{
  char DataSum;

  tmp_data[0] = MCP9600_Status;                                                 // Register address
  
  I2CM_Set_Slave_Address( MCP9600_I2C_ADDR );
  I2CM_Write_Bytes( tmp_data, 1 );
  Delay_us(50);
  I2CM_Read_Bytes( tmp_data, 1 );
  DataSum = tmp_data[0];

  return DataSum;                                                               // Return MCP9600 Data value
}

/*******************************************************************************
* Function Read_Temperature()
* ------------------------------------------------------------------------------
* Overview: Function gets Thermocouple Temperature from MCP9600
* Input: Nothing
* Output: Thermocouple Temperature
*******************************************************************************/
float Read_Temperature()
{
  float Temperature;

  tmp_data[0] = MCP9600_TH;                                                     // Thermocouple Temperature register

  I2CM_Set_Slave_Address( MCP9600_I2C_ADDR );
  I2CM_Write_Bytes( tmp_data, 1 );
  Delay_us(50);
  I2CM_Read_Bytes( tmp_data, 2 );

  if((tmp_data[0] & 0x80) == 0x80){                                             // TA < 0C
    tmp_data[0] = tmp_data[0] & 0x7F;                                           // Clear SIGN
    Temperature = 1024 - (tmp_data[0]*16 + tmp_data[1] / 16);
  }
  else                                                                          // TA > 0C
    Temperature = (tmp_data[0] * 16 + (float)tmp_data[1] / 16);                 // Temperature = Ambient Temperature (C)

  return Temperature;                                                           // Return Thermocouple temperature data
}

/*******************************************************************************
* Function updateLabel(char *newText, int x, int y, char* lbl)
* ------------------------------------------------------------------------------
* Overview: Function writes text on display and erase old text
* Input: Nothing
* Output: Updated text
*******************************************************************************/
void updateLabel(char *newText, int x, int y, char* lbl)
{
  TFT_Set_Font(&Tahoma52x52_Bold, CL_WHITE, FO_HORIZONTAL);                     // Color of background
  TFT_Write_Text(lbl, x, y);                                                    // Write text on TFT display with background color
  strcpy(lbl, newText);                                                         // Copy newText to lbl
  TFT_Set_Font(&Tahoma52x52_Bold, CL_BLACK, FO_HORIZONTAL);                     // Color of text
  TFT_Write_Text(lbl, x, y);                                                    // Write text on  TFT display with black color
}

//******************************************************************************
// Main function
//******************************************************************************
void main()
{
  char res;

  display_init();                                                               // Initialize display
  TFT_Image(20, 82, Thermometer_bmp, 1);

  // Initialize the I2C1 module with 100Kbps speed on PORTB
  I2CM1_Init( _I2CM_SPEED_MODE_STANDARD, _I2CM_SWAP_DISABLE );
  Delay_ms(200);                                                                // Delay 200 ms

  while(1){
    tk_status = &Read_MCP9600_Status;
    Temp_Data_Ready = ( tk_status & 0x40 );

    if (Temp_Data_Ready){                                                       // Check is Temperature Data ready
      sprintf(txt, "%2.1fC", Read_Temperature());                              // Format Temperature Data and store it to txt
      res = strcmp(txt,old_txt);                                                // Compare old_txt and txt

      if(res != 0){                                                             // If old_txt and txt are not equal
        updateLabel(txt, 135, 90, lab1);                                        // Write temperature value on display
        strcpy(old_txt,txt);                                                    // Copy txt to old_txt string array
      }
    }
    Delay_ms(500);
  }
}
